@@ -735,6 +735,7 @@ process_message (MbimDevice        *self,
     case MBIM_MESSAGE_TYPE_CLOSE:
     case MBIM_MESSAGE_TYPE_COMMAND:
     case MBIM_MESSAGE_TYPE_HOST_ERROR:
+    default:
         /* Shouldn't expect host-generated messages as replies */
         g_message ("[%s] Host-generated message received: ignoring",
                    self->priv->path_display);
@@ -1603,7 +1604,7 @@ device_open_context_step (GTask *task)
         self->priv->open_status = OPEN_STATUS_OPENING;
 
         ctx->step++;
-        /* Fall down */
+        /* Fall through */
 
     case DEVICE_OPEN_CONTEXT_STEP_CREATE_IOCHANNEL:
         create_iochannel (self,
@@ -1618,7 +1619,7 @@ device_open_context_step (GTask *task)
             return;
         }
         ctx->step++;
-        /* Fall down */
+        /* Fall through */
 
     case DEVICE_OPEN_CONTEXT_STEP_CLOSE_MESSAGE:
         /* Only send an explicit close during open if needed */
@@ -1628,7 +1629,7 @@ device_open_context_step (GTask *task)
             return;
         }
         ctx->step++;
-        /* Fall down */
+        /* Fall through */
 
     case DEVICE_OPEN_CONTEXT_STEP_OPEN_MESSAGE:
         /* If the device is already in-session, avoid the open message */
@@ -1637,7 +1638,7 @@ device_open_context_step (GTask *task)
             return;
         }
         ctx->step++;
-        /* Fall down */
+        /* Fall through */
 
     case DEVICE_OPEN_CONTEXT_STEP_LAST:
         /* Nothing else to process, complete without error */
@@ -1971,6 +1972,10 @@ device_write (MbimDevice    *self,
             /* We're in a non-blocking channel and therefore we're up to receive
              * EAGAIN; just retry in this case. TODO: in an idle? */
             break;
+
+        default:
+            g_assert_not_reached ();
+            break;
         }
     }
 
