@@ -25,6 +25,7 @@
 #include "mbim-message.h"
 #include "mbim-cid.h"
 #include "mbim-common.h"
+#include "mbim-error-types.h"
 
 #if defined ENABLE_TEST_MESSAGE_TRACES
 static void
@@ -661,6 +662,41 @@ test_message_parser_basic_connect_register_state (void)
     mbim_message_unref (response);
 }
 
+static void
+test_message_parser_provisioned_contexts (void)
+{
+    MbimMessage *response;
+    guint32 provisioned_contexts_count = 0;
+    MbimProvisionedContextElement **provisioned_contexts = NULL;
+    GError *error = NULL;
+    const guint8 buffer [] = {
+        /* header */
+        0x03, 0x00, 0x00, 0x80, /* type */
+        0x30, 0x00, 0x00, 0x00, /* length */
+        0x1C, 0x00, 0x00, 0x00, /* transaction id */
+        /* fragment header */
+        0x01, 0x00, 0x00, 0x00, /* total */
+        0x00, 0x00, 0x00, 0x00, /* length */
+        /* command_done_message */
+        0xA2, 0x89, 0xCC, 0x33, /* service id */
+        0xBC, 0xBB, 0x8B, 0x4F,
+        0xB6, 0xB0, 0x13, 0x3E,
+        0xC2, 0xAA, 0xE6, 0xDF,
+        0x0D, 0x00, 0x00, 0x00, /* command id */
+        0x00, 0x00, 0x00, 0x00, /* status code */
+        0x00, 0x00, 0x00, 0x00  /* buffer length */
+    };
+
+    response = mbim_message_new (buffer, sizeof (buffer));
+    g_assert (!mbim_message_provisioned_contexts_response_parse (
+                  response,
+                  &provisioned_contexts_count,
+                  &provisioned_contexts,
+                  &error));
+
+    g_assert_error (error, MBIM_CORE_ERROR, MBIM_CORE_ERROR_INVALID_MESSAGE);
+}
+
 static void
 test_message_parser_sms_read_zero_pdu (void)
 {
@@ -1185,7 +1221,7 @@ test_message_parser_stk_pac_response (void)
         0x38, 0x16, 0x16, 0x4C,
         0x01, 0x00, 0x00, 0x00, /* command id */
         0x00, 0x00, 0x00, 0x00, /* status code */
-        0x00, 0x01, 0x00, 0x00, /* buffer length */
+        0x00, 0x01, 0x00, 0x00, /* buffer length (256) */
         /* information buffer */
         0x04, 0x05, 0x06, 0x07,
         0xAF, 0xAE, 0xAD, 0xAC,
@@ -1226,9 +1262,34 @@ test_message_parser_stk_pac_response (void)
         0x7B, 0x7A, 0x79, 0x78,
         0x77, 0x76, 0x75, 0x74,
         0x73, 0x72, 0x00, 0x00,
+        0x01, 0x00, 0x00, 0x00,
+        0x04, 0x05, 0x06, 0x07,
+        0xAF, 0xAE, 0xAD, 0xAC,
+        0xAB, 0xAA, 0xA9, 0xA8,
+        0xA7, 0xA6, 0xA5, 0xA4,
+        0xA3, 0xA2, 0xA1, 0xA0,
+        0x7F, 0x7E, 0x7D, 0x7C,
+        0x7B, 0x7A, 0x79, 0x78,
+        0x77, 0x76, 0x75, 0x74,
+        0x73, 0x72, 0x00, 0x00,
+        0x01, 0x00, 0x00, 0x00,
+        0x04, 0x05, 0x06, 0x07,
+        0xAF, 0xAE, 0xAD, 0xAC,
+        0xAB, 0xAA, 0xA9, 0xA8,
+        0xA7, 0xA6, 0xA5, 0xA4,
+        0xA3, 0xA2, 0xA1, 0xA0,
+        0x7F, 0x7E, 0x7D, 0x7C,
+        0x7B, 0x7A, 0x79, 0x78,
+        0x77, 0x76, 0x75, 0x74,
+        0x73, 0x72, 0x00, 0x00,
+        0x01, 0x00, 0x00, 0x00,
+        0x04, 0x05, 0x06, 0x07,
+        0xAF, 0xAE, 0xAD, 0xAC,
+        0xAB, 0xAA, 0xA9, 0xA8,
+        0xA7, 0xA6, 0xA5, 0xA4,
     };
 
-    const guint8 expected_databuffer [] = {
+    const guint8 expected_databuffer [256] = {
         0x04, 0x05, 0x06, 0x07,
         0xAF, 0xAE, 0xAD, 0xAC,
         0xAB, 0xAA, 0xA9, 0xA8,
@@ -1267,7 +1328,32 @@ test_message_parser_stk_pac_response (void)
         0x7F, 0x7E, 0x7D, 0x7C,
         0x7B, 0x7A, 0x79, 0x78,
         0x77, 0x76, 0x75, 0x74,
-        0x73, 0x72, 0x00, 0x00
+        0x73, 0x72, 0x00, 0x00,
+        0x01, 0x00, 0x00, 0x00,
+        0x04, 0x05, 0x06, 0x07,
+        0xAF, 0xAE, 0xAD, 0xAC,
+        0xAB, 0xAA, 0xA9, 0xA8,
+        0xA7, 0xA6, 0xA5, 0xA4,
+        0xA3, 0xA2, 0xA1, 0xA0,
+        0x7F, 0x7E, 0x7D, 0x7C,
+        0x7B, 0x7A, 0x79, 0x78,
+        0x77, 0x76, 0x75, 0x74,
+        0x73, 0x72, 0x00, 0x00,
+        0x01, 0x00, 0x00, 0x00,
+        0x04, 0x05, 0x06, 0x07,
+        0xAF, 0xAE, 0xAD, 0xAC,
+        0xAB, 0xAA, 0xA9, 0xA8,
+        0xA7, 0xA6, 0xA5, 0xA4,
+        0xA3, 0xA2, 0xA1, 0xA0,
+        0x7F, 0x7E, 0x7D, 0x7C,
+        0x7B, 0x7A, 0x79, 0x78,
+        0x77, 0x76, 0x75, 0x74,
+        0x73, 0x72, 0x00, 0x00,
+        0x01, 0x00, 0x00, 0x00,
+        0x04, 0x05, 0x06, 0x07,
+        0xAF, 0xAE, 0xAD, 0xAC,
+        0xAB, 0xAA, 0xA9, 0xA8,
+        0xA7, 0xA6, 0xA5, 0xA4,
     };
 
     response = mbim_message_new (buffer, sizeof (buffer));
@@ -1694,6 +1780,53 @@ test_message_parser_ms_firmware_id_get (void)
     mbim_message_unref (response);
 }
 
+static void
+test_message_parser_basic_connect_connect_short (void)
+{
+    MbimMessage *response;
+    GError *error = NULL;
+    guint32 session_id;
+    MbimActivationState activation_state;
+    MbimVoiceCallState voice_call_state;
+    MbimContextIpType ip_type;
+    const MbimUuid *context_type;
+    guint32 nw_error;
+    const guint8 buffer [] = {
+        /* header */
+        0x03, 0x00, 0x00, 0x80, /* type */
+        0x30, 0x00, 0x00, 0x00, /* length */
+        0x1A, 0x0D, 0x00, 0x00, /* transaction id */
+        /* fragment header */
+        0x01, 0x00, 0x00, 0x00, /* total */
+        0x00, 0x00, 0x00, 0x00, /* current */
+        /* command_done_message */
+        0xA2, 0x89, 0xCC, 0x33, /* service id */
+        0xBC, 0xBB, 0x8B, 0x4F,
+        0xB6, 0xB0, 0x13, 0x3E,
+        0xC2, 0xAA, 0xE6, 0xDF, /* command id */
+        0x0C, 0x00, 0x00, 0x00, /* status code */
+        0x02, 0x00, 0x00, 0x00, /* buffer length */
+        /* information buffer */
+        0x00, 0x00, 0x00, 0x00
+    };
+
+    response = mbim_message_new (buffer, sizeof (buffer));
+
+    /* should fail! */
+    g_assert (!mbim_message_connect_response_parse (
+                  response,
+                  &session_id,
+                  &activation_state,
+                  &voice_call_state,
+                  &ip_type,
+                  &context_type,
+                  &nw_error,
+                  &error));
+
+    g_assert (error != NULL);
+    g_error_free (error);
+}
+
 int main (int argc, char **argv)
 {
     g_test_init (&argc, &argv, NULL);
@@ -1704,6 +1837,7 @@ int main (int argc, char **argv)
     g_test_add_func ("/libmbim-glib/message/parser/basic-connect/ip-configuration", test_message_parser_basic_connect_ip_configuration);
     g_test_add_func ("/libmbim-glib/message/parser/basic-connect/service-activation", test_message_parser_basic_connect_service_activation);
     g_test_add_func ("/libmbim-glib/message/parser/basic-connect/register-state", test_message_parser_basic_connect_register_state);
+    g_test_add_func ("/libmbim-glib/message/parser/basic-connect/provisioned-contexts", test_message_parser_provisioned_contexts);
     g_test_add_func ("/libmbim-glib/message/parser/sms/read/zero-pdu", test_message_parser_sms_read_zero_pdu);
     g_test_add_func ("/libmbim-glib/message/parser/sms/read/single-pdu", test_message_parser_sms_read_single_pdu);
     g_test_add_func ("/libmbim-glib/message/parser/sms/read/multiple-pdu", test_message_parser_sms_read_multiple_pdu);
@@ -1717,6 +1851,7 @@ int main (int argc, char **argv)
     g_test_add_func ("/libmbim-glib/message/parser/basic-connect/ip-packet-filters/one", test_message_parser_basic_connect_ip_packet_filters_one);
     g_test_add_func ("/libmbim-glib/message/parser/basic-connect/ip-packet-filters/two", test_message_parser_basic_connect_ip_packet_filters_two);
     g_test_add_func ("/libmbim-glib/message/parser/ms-firmware-id/get", test_message_parser_ms_firmware_id_get);
+    g_test_add_func ("/libmbim-glib/message/parser/basic-connect/connect/short", test_message_parser_basic_connect_connect_short);
 
     return g_test_run ();
 }
