@@ -142,7 +142,6 @@ log_handler (const gchar *log_domain,
         break;
 
     case G_LOG_LEVEL_CRITICAL:
-    case G_LOG_FLAG_FATAL:
     case G_LOG_LEVEL_ERROR:
         log_level_str = "-Error **";
         err = TRUE;
@@ -152,9 +151,16 @@ log_handler (const gchar *log_domain,
         log_level_str = "[Debug]";
         break;
 
-    default:
+    case G_LOG_LEVEL_MESSAGE:
+    case G_LOG_LEVEL_INFO:
         log_level_str = "";
         break;
+
+    case G_LOG_FLAG_FATAL:
+    case G_LOG_LEVEL_MASK:
+    case G_LOG_FLAG_RECURSION:
+    default:
+        g_assert_not_reached ();
     }
 
     if (!verbose_flag && !err)
@@ -167,12 +173,13 @@ log_handler (const gchar *log_domain,
                message);
 }
 
+G_GNUC_NORETURN
 static void
 print_version_and_exit (void)
 {
     g_print ("\n"
              PROGRAM_NAME " " PROGRAM_VERSION "\n"
-             "Copyright (C) 2013-2019 Aleksander Morgado\n"
+             "Copyright (C) 2013-2020 Aleksander Morgado\n"
              "License GPLv2+: GNU GPL version 2 or later <http://gnu.org/licenses/gpl-2.0.html>\n"
              "This is free software: you are free to change and redistribute it.\n"
              "There is NO WARRANTY, to the extent permitted by law.\n"
@@ -281,6 +288,14 @@ device_open_ready (MbimDevice   *dev,
     case MBIM_SERVICE_MS_BASIC_CONNECT_EXTENSIONS:
         mbimcli_ms_basic_connect_extensions_run (dev, cancellable);
         return;
+    case MBIM_SERVICE_SMS:
+    case MBIM_SERVICE_USSD:
+    case MBIM_SERVICE_STK:
+    case MBIM_SERVICE_AUTH:
+    case MBIM_SERVICE_PROXY_CONTROL:
+    case MBIM_SERVICE_QMI:
+        /* unsupported actions in the CLI */
+    case MBIM_SERVICE_INVALID:
     default:
         g_assert_not_reached ();
     }
