@@ -36,6 +36,7 @@ GOptionGroup *mbimcli_ms_firmware_id_get_option_group   (void);
 GOptionGroup *mbimcli_ms_host_shutdown_get_option_group (void);
 GOptionGroup *mbimcli_atds_get_option_group             (void);
 GOptionGroup *mbimcli_intel_firmware_update_get_option_group (void);
+GOptionGroup *mbimcli_ms_basic_connect_extensions_get_option_group (void);
 
 gboolean      mbimcli_basic_connect_options_enabled     (void);
 gboolean      mbimcli_phonebook_options_enabled         (void);
@@ -44,6 +45,7 @@ gboolean      mbimcli_ms_firmware_id_options_enabled    (void);
 gboolean      mbimcli_ms_host_shutdown_options_enabled  (void);
 gboolean      mbimcli_atds_options_enabled              (void);
 gboolean      mbimcli_intel_firmware_update_options_enabled (void);
+gboolean      mbimcli_ms_basic_connect_extensions_options_enabled (void);
 
 void          mbimcli_basic_connect_run                 (MbimDevice *device,
                                                          GCancellable *cancellable);
@@ -59,5 +61,7 @@ void          mbimcli_atds_run                          (MbimDevice *device,
                                                          GCancellable *cancellable);
 void          mbimcli_intel_firmware_update_run         (MbimDevice *device,
                                                          GCancellable *cancellable);
+void          mbimcli_ms_basic_connect_extensions_run   (MbimDevice *device,
+                                                         GCancellable *cancellable);
 
 #endif /* __MBIMCLI_H__ */
